/* @stubs.c - stub functions for unsupported functions
 */
#include <stdio.h>
#include <errno.h>
#include <malloc.h>
#include <sys/stat.h>
#include "gen.h"

#ifdef NEVER
int ustat(char *path, Stat *sstat)
	{
	return stat(path, sstat);
	}
#endif
#if defined(XENIX) || defined(_POSIX_SOURCE) || defined(DOS) || defined(sparc)
int chsize(FD fd, Seek size)
	{
	int rtn = 0;
	struct stat sstat;
	Seek save;
	char *buf = NULL;
	int wsize;

#	ifdef DEB
	errfmt("chsize: stub\n");
#	endif
	if (fstat(fd, &sstat) < 0)
		rtn = -ERRNO;
	else if ((buf = malloc(BUFSIZ)) == NULL)
		rtn = -ERRNO;
	else if ((save = lseek(fd, 0L, 1)) < 0)
		rtn = -ERRNO;
	else if (sstat.st_size < size)
		{
#		ifdef DEB
		errfmt("chsize: <<<\n");
#		endif
		if (lseek(fd, sstat.st_size, 0) < 0)
			rtn = -ERRNO;
		else
			{
#		ifdef DEB
		errfmt("chsize: 2\n");
#		endif
			memset(buf, '\0', BUFSIZ);
			while (sstat.st_size < size)
				{
				wsize =  min(BUFSIZ, size - sstat.st_size);
				if (write(fd, buf, wsize) != wsize)
					{
					rtn = -ERRNO;
					break;
					}
				sstat.st_size += wsize;
				}
			}
		}
	else if (lseek(fd, save, 0) < 0)
		rtn = -ERRNO;
	if (buf != NULL)
		free(buf);
	return rtn;
	}
#endif

#if defined(XENIX) || defined(_POSIX_SOURCE) || defined(DOS)
int symlink(char *from, char *to)
	{
	return -(errno = EXDEV);
	}
int lstat(char *path, Stat *sstat)
	{
	return stat(path, sstat);
	}
int readlink(char *path, char *buf, int bufsiz)
	{
	return -(errno = EINVAL);
	}

#endif
#if defined(_POSIX_SOURCE) || defined (DOS)
int statfs(char *path, Statfs *sstatfs, int len, int fstype)
	{
	errno = EINVAL;
	return -ERRNO;
	}
int fstatfs(FD fd, Statfs *sstatfs, int len, int fstype)
	{
	errno = EINVAL;
	return -ERRNO;
	}
int mknod(char const *path, Mode mode, dev_t dev)
	{
	errno = EINVAL;
	return -ERRNO;
	}
#endif
#if defined(_POSIX_SOURCE) 
int putenv(char *string)
	{
	return -EINVAL;
	}
#endif

#if defined(DOS)

int setuid(Uid uuid)
	{
	return 0;
	}
pid_t wait(int *loc)
	{
	return 0;
	}
int link(char *from, char *to)
	{
	errno = EMLINK;
	return -ERRNO;
	}
Gid getegid(void)
	{
	return 0;
	}
int fork(void)
	{
	return 0;
	}
Uid geteuid(void)
	{
	return 0;
	}
int chown(char *path, Uid uuid, Gid gid)
	{
	return 0;
	}
Uid getuid(void)
	{
	return 0;
	}

#ifdef NEVER
int locking(FD fd, int mode, long size)
	{
	return 0;
	}
#endif

#endif
#ifdef NEVER
int tsymlink(char *path, char *newpath)
	{
	SD sd;
	FD fd;
	Mode oumask;
	static short levels[DIRLEVELS] = {DIRECTB, INDIRECTB, DINDIRECTB, TINDIRECTB};
	newfs(newpath, 4, 256, levels, 0);

	if (0 <= (sd = fsopen(path)))
		{
		oumask = xumask(0);
		if (0 <= (fd = iopen(sd, ROOTINODE, O_CREAT|O_WRONLY, 0777|S_IFLNK)))
			{
			xwrite(fd, path, strlen(path));
			xclose(fd);
			}
		xumask(oumask);
		fsclose(sd);
		}
	return 0;
	}
#endif
