#include <gen.h> 
#include <xalloc.h>
#include "hash.h"
#include "general.h"

extern char * bhome;

typedef int ChapterNum, VerseNum, AbsVerse, RangeNum, BookmarkNum;
typedef int Line, Pos, BookNo;

#define noChapterNum -1
#define noVerseNum -1

typedef struct
	{
	VerseNum numVerses;
	AbsVerse begVerse;
	} Chapter;

typedef struct
	{
	char * str;
    int num;
	} StringIntHash;

typedef struct
	{
	Bool textLoaded;
	Bool infoLoaded;
	char * data;
	Pos size;  
	Pos * verses;
	AbsVerse numVerses;
	Pos cPos;
	char * title;
	Chapter * chapters;
	ChapterNum numChapters;
    } Book;

typedef struct
    {
	char * title;
	Book * cBook;
	Book * books;
    HashTable bookNameHash;
    StringIntHash * bookNameHashItems;
	BookNo numBooks;
    } Version;

typedef struct
    {
	char * title;
	ChapterNum chap;
	VerseNum verse;
    } Verse;

typedef struct
	{
	Verse * list;
	int num;
	int arraySize;
	} VerseList;

typedef struct
    {
	Verse from;
	Verse to;
    } RangeItem;

typedef struct
    {
	RangeItem * items;
	RangeNum num;
    } Range;

typedef struct
    {
	char * title;
	Range * range;
    } Bookmark;

typedef struct
    {
	Bookmark * bookmarks;
    HashTable bookmarkHash;
    StringIntHash * bookmarkHashItems;
	int num;
    } BookmarkList;

typedef struct context
	{
	int (*run)(struct context *);
	Version * cVersion;
    } Context;

#define ERRBUF_SIZE 1024
#define RANGEBUF_SIZE 1024
typedef char ErrorBuf[ERRBUF_SIZE];

extern Version * cVersion;
extern BookmarkList * bmarklist;

#define MAT_NOMATCH -1
#define MAT_ERROR -2

#define BOOKNAME_SIZE 512
#define VERSESTRING_SIZE 512

#define skipwhite(str) while (isspace(*str)) str++

#ifdef NO_MEMMOVE
#define memmove(t,f,s) bcopy(f,t,s)
#endif

void init();
char ** getvector(char * filename, int * size);
void freevector(char ** vec);
void loadBook(Version * version, Book * book, Bool data);
void freeBook(Book * book);
void getVersionListFileName(char * filename);
void getBookContentsFileName(char * fileName, char * version);
Version * loadBookVersion(char * version, char * book);
AbsVerse searchNextBook(Version * version, Book * book, AbsVerse pos, Range * range, char * expression, ErrorBuf mesg, Bool back);
AbsVerse searchNextVersion(Version * version, AbsVerse pos, Range * range, char * expression, ErrorBuf mesg, Bool back, Bool * newBook);
int whatVerse(Book * book, int pos);
int verseLength(register char * verse);
Pos endOfVerse(Book * book, AbsVerse verse);
AbsVerse nextSearchVerse(Book * book, Pos pos);
AbsVerse prevSearchVerse(Book * book, Pos pos);
void parseVerseLine(char * verse, ChapterNum * chap, VerseNum * verseNum);
Range * parseVerse(char * verse, int * eaten);
Pos verseToPos(Book * book, AbsVerse ver);
AbsVerse gotoVerseString(Version * ver, char * verse, Bool * nBook);
AbsVerse gotoVerse(Version * ver, char * book, ChapterNum chap, VerseNum verseNum, Bool * nBook);
BookmarkList * getBookmarks(char * filename);
BookmarkList * combineBookmarks(BookmarkList * l1, BookmarkList * l2);
void freeBookmarks(BookmarkList * list);
BookmarkList * reopenBookmark(char * filename, BookmarkList * old);
BookmarkList * combinedLoadBookmark(char * filename, BookmarkList * old);
Version * loadVersion(char * version);
void loadDefaultBook(Version * ver);
Bookmark * markFromList(BookmarkList * list, char * name);
char * rangeToBuf(Range * range, char * buf);
Version * findVersion(char * title);
char ** getVersionListSVec(int * num);
char ** getBookContentsSVec(char * ver, int * num);
void normaliseFromVerse(Verse * verse);
void freeBookmark(Bookmark * bmark);
void bookmarkAdd(BookmarkList * bl, Bookmark * bm, BookmarkNum pos);
BookmarkList * bookmark_create(BookmarkNum size);
VerseList * searchVersionReturnAll(Version * version, Range * range, char * expression, ErrorBuf mesg);
